/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.tracer.impl;

//=============================================================================

import jatva.tracer.TracerException;

import java.io.File;
import java.util.Properties;

//=============================================================================

/** 
 * This class implements writing trace messages to a file in the given directory.
 * TODO: Write more detailed documentation for this class.
 *
 * @author  <a href="mailto:barancev@ispras.ru">Alexei Barantsev</a>
 * @version 1.0
 */
public class WriteToDirectory extends AbstractWriter {

    /** 
     * Name of configuration property that specifies directory where writer
     * should create file to write trace to.
     */
    public static String DIRECTORY_PROPERTY = "directory";

    public static String TRACE_FILE_EXTENTION = ".utt";

    private File directory;

    private WriteToFile fileWriter;

    /** 
     * Creates a new non-configured instance of <code>WriteToDirectory</code>.<br/>
     * <strong>WARNING:</strong> This constructor creates non-configured
     * instance that can't be used immediately. It is designed to be used by
     * <code>TracerConfiguration</code> only. For other purposes use more
     * specific constructors.
     */
    public WriteToDirectory() {
    }

    /** 
     * Creates a new instance of <code>WriteToDirectory</code> that writes
     * traces to files in the specified directory.
     * 
     * @param directory Directory where trace files should be put to.
     */
    public WriteToDirectory(File directory) {
        this.directory = directory;
    }

    public WriteToDirectory clone() {
        WriteToDirectory newObj = (WriteToDirectory) super.clone();
        newObj.fileWriter = null;
        return newObj;
    }

    public void configure(Properties writerProperties) {
        String dirProp = writerProperties.getProperty(DIRECTORY_PROPERTY);
        
        if (dirProp == null) {
            throw new TracerException("Directory not specified");

        } else {
            setDirectory(new File(dirProp));
        }
    }
    
    private File getDirectory() {
        return directory;
    }

    private synchronized void setDirectory(File directory) {
        if (isConfigurationChangeAllowed()){
            this.directory = directory;

        } else {
            System.err.println("WARNING: tracer configuration cannot be changed - tracing already started");
        }
    }

    public synchronized void prepare()
    {
        if (fileWriter == null) {
            if(getDirectory().exists() && !getDirectory().isDirectory()) {
                throw new TracerException("" + getDirectory() + " is not a directory");
            }

            if (!getDirectory().exists()) {
                try {
                    if(!getDirectory().mkdirs()) {
                        throw new TracerException("Could not create directory " + getDirectory());
                    }

                } catch (SecurityException e) {
                    throw new TracerException("Could not create directory " + getDirectory(), e);
                }
            }

            fileWriter = new WriteToFile(
                new File(getDirectory(), getTestId() + TRACE_FILE_EXTENTION));

            fileWriter.setTestId( this.getTestId() );

            fileWriter.prepare();
        }
        setTracingInProgress( true );
    }

    public void sendMessage( StringBuffer message )
    {
        fileWriter.sendMessage(message);
    }

    public void finish() 
    {
        setTracingInProgress( false );

        if (fileWriter != null) {
            fileWriter.finish();
            fileWriter = null;
        }
    }

}
//=============================================================================
// End of file WriteToDirectory.java
