/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.tracer.impl;

//=============================================================================

import jatva.tracer.TracerException;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Properties;

//=============================================================================

/** 
 * This class implements writing trace messages to network monitor.
 * TODO: Write more detailed documentation for this class.
 *
 * @author  <a href="mailto:barancev@ispras.ru">Alexei Barantsev</a>
 * @version 1.0
 */
public class WriteToMonitor extends AbstractWriter {

    /** 
     * Name of configuration property that specifies monitor server host.
     */
    public static String ADDRESS_PROPERTY = "address";

    /** 
     * Name of configuration property that specifies monitor server port.
     */
    public static String PORT_PROPERTY = "port";

    /** 
     * Negotiation message prefix.
     */
    public static String NEGOTIATION_PREFIX = "@NEGOTIATION: ";

    /** 
     * Negotiation request to allocate new test id.
     */
    public static String NEGOTIATION_GET_TESTID = NEGOTIATION_PREFIX  + "GET TESTID";

    /** 
     * Negotiation message that inform monitor about test id.
     */
    public static String NEGOTIATION_SET_TESTID = NEGOTIATION_PREFIX  + "SET TESTID ";

    /** 
     * Negotiation request to allocate new scenario id.
     */
    public static String NEGOTIATION_GET_SCID = NEGOTIATION_PREFIX  + "GET SCID";

    /** 
     * Default monitor server port.
     */
    public static int defaultServerPort = 8070;

    /** 
     * Holds address of monitor server.
     */
    private InetAddress serverAddress;

    /** 
     * Holds port of monitor server.
     */
    private int serverPort = defaultServerPort;

    /** 
     * Holds socket connection to monitor.
     */
    private Socket socket;

    /** 
     * Holds writer to socket connection to monitor.
     */
    private BufferedWriter writer;
    
    /** 
     * Holds reader from socket connection to monitor.
     */
    private BufferedReader reader;
    
    /** 
     * Creates a new non-configured instance of <code>WriteToMonitor</code>.<br/>
     * <strong>WARNING:</strong> This constructor creates non-configured
     * instance that can't be used immediately. It is designed to be used by
     * <code>TracerConfiguration</code> only. For other purposes use more
     * specific constructors.
     */
    public WriteToMonitor() {
    }

    /** 
     * Creates a new instance of <code>WriteToMonitor</code> that writes trace
     * to the monitor located on the specified address and listening to the
     * specified port.
     * 
     * @param address Trace monitor address.
     * @param port Port listened by trace monitor.
     */
    public WriteToMonitor(InetAddress address, int port) {
        serverAddress = address;
        serverPort = port;
    }

    public synchronized WriteToMonitor clone() {
        WriteToMonitor newObj = (WriteToMonitor) super.clone();
        newObj.socket = null;
        newObj.writer = null;
        return newObj;
    }

    public void configure(Properties writerProperties) {
        String hostName = writerProperties.getProperty(ADDRESS_PROPERTY);
        try {
            setServerAddress(InetAddress.getByName(hostName));

        } catch (UnknownHostException e) {
            throw new TracerException("Unknown host " + hostName, e);

        } catch (SecurityException e) {
            throw new TracerException("Could not resolve host name " + hostName, e);
        }

        String port = writerProperties.getProperty(PORT_PROPERTY);
        if (port != null) {
            try {
                setServerPort(Integer.parseInt(port));

            } catch (NumberFormatException e) {
                throw new TracerException("Bad port value specified: " + port, e);
            }
        }
    }

    private InetAddress getServerAddress() {
        return serverAddress;
    }

    private synchronized void setServerAddress(InetAddress serverAddress) {
        if (isConfigurationChangeAllowed()){
            this.serverAddress = serverAddress;

        } else {
            System.err.println("WARNING: tracer configuration cannot be changed - tracing already started");
        }
    }

    private int getServerPort() {
        return serverPort;
    }

    private synchronized void setServerPort(int serverPort) {
        if (isConfigurationChangeAllowed()){
            this.serverPort = serverPort;

        } else {
            System.err.println("WARNING: tracer configuration cannot be changed - tracing already started");
        }
    }

    private Socket getSocket() {
        if (socket == null) {
            try {
                socket = new Socket(getServerAddress(), getServerPort());

            } catch (IOException e) {
                throw new TracerException(
                		"Could not establish connection to monitor at "
                        + "[addr=" + getServerAddress()
                        + ",port=" + getServerPort() + "]",
                        e);

            } catch (SecurityException e) {
                throw new TracerException(
                		"Could not establish connection to monitor at "
                        + "[addr=" + getServerAddress()
                        + ",port=" + getServerPort() + "]",
                        e);
            }
        }

        return socket;
    }

    private BufferedWriter getWriter() {
        if (writer == null) {
            try {
                writer = new BufferedWriter(new OutputStreamWriter(getSocket().getOutputStream()));

            } catch (IOException e) {
                throw new TracerException("Could not open socket for writing", e);
            }
        }

        return writer;
    }

    private BufferedReader getReader() {
        if (reader == null) {
            try {
                reader = new BufferedReader(new InputStreamReader(getSocket().getInputStream()));

            } catch (IOException e) {
                throw new TracerException("Could not open socket for reading", e);
            }
        }

        return reader;
    }

    public void sendMessage(StringBuffer message) {

        if (!isTracingInProgress() && !message.toString().startsWith(NEGOTIATION_PREFIX)) {
            sendMessage(new StringBuffer(NEGOTIATION_SET_TESTID + getTestId()));
            setTracingInProgress(true);
        }

        try {
            BufferedWriter writer = getWriter();

            writer.write(message.toString().trim());
            writer.newLine();
            writer.flush();

        } catch (IOException e) {
            throw new TracerException("Could not write to socket", e);
        }
    }

    public void finish() {
        super.finish();

        if (socket != null) {
            try {
                socket.shutdownOutput();
    
            } catch (IOException e) {
                throw new TracerException("Could not close connection", e);

            } finally {
                socket = null;
                writer = null;
            }
        }
    }

    /** Requests unique test id from monitor.
     * @see jatva.tracer.impl.AbstractWriter#allocateTestId()
     */
    public String allocateTestId() {
        sendMessage(new StringBuffer(NEGOTIATION_GET_TESTID));

        try {
            return getReader().readLine();

        } catch (IOException e) {
            throw new TracerException("Could not get unique test id from monitor", e);
        } 
    }

    /** Requests unique scenario id from monitor. 
     * @see jatva.tracer.impl.AbstractWriter#allocateWrid()
     */
    public int allocateWrid() {
        sendMessage(new StringBuffer(NEGOTIATION_GET_SCID));

        try {
            return Integer.parseInt(getReader().readLine().trim());

        } catch (IOException e) {
            throw new TracerException("Could not get unique scenario id from monitor", e);

        } catch (NumberFormatException e) { 
            throw new TracerException("Could not get unique scenario id from monitor", e);
        }
    }

}
//=============================================================================
// End of file WriteToMonitor.java
